"""coinsta - A Python package for acquiring both historical and current data of cryptocurrencies"""


from .core import Historical, Current
from .exceptions import WrongCoinCode

__version__ = '0.1.2'
__author__ = 'Bernard Brenyah <bbrenyah@gmail.com>'
__all__ = []



import unittest
import warnings
from datetime import date
from coinsta.exceptions import WrongCoinCode
from coinsta.core import Historical, Current


class TestCoinsta(unittest.TestCase):

    def test_historical(self):

        with self.assertRaises(TypeError):
            start = date(2018, 2, 20)
            return Historical('dash', start=start, end=123)

    def test_historical_start(self):

        with self.assertRaises(TypeError):
            end = date(2018, 2, 20)
            return Historical('dash', start=123, end=end)

    def test_historical_ticker(self):

        with self.assertRaises(WrongCoinCode):
            start = date(2018, 2, 20)
            return Historical('fake_ticker', start=start).get_data()

    def test_get_data(self):
        warnings.filterwarnings("ignore", message="can't resolve package from __spec__ or __package__, "
                                                  "falling back on __name__ and __path__")

        coin_spec = Historical.from_strings('dash', '2018-1-1', '2018-3-1', hyphen=True)
        data = coin_spec.get_data()
        data_cols = len(data.columns)

        self.assertEqual(data_cols, 6)

    def test_from_strings(self):
        alt_spec = Historical.from_strings('xrp', '2018-1-1', '2018-3-1', hyphen=True)
        start = alt_spec.start
        end = alt_spec.end
        ticker = alt_spec.ticker

        self.assertEqual(start, '20180101')
        self.assertEqual(end, '20180301')
        self.assertEqual(ticker, 'xrp')

    def test_from_strings_2(self):
        alt_spec = Historical.from_strings('btc', '2018/1/1', '2018/3/1', hyphen=False)
        start = alt_spec.start
        end = alt_spec.end
        ticker = alt_spec.ticker

        self.assertEqual(start, '20180101')
        self.assertEqual(end, '20180301')
        self.assertEqual(ticker, 'btc')

    def test_get_current(self):
        btc_current = Current.get_current('btc')
        no_items = len(btc_current.index)
        self.assertEqual(no_items, 12)

    def test_global_info(self):
        glo_info = Current.global_info()
        no_keys = len(glo_info.keys())
        self.assertEqual(no_keys, 5)

    def test_top_100(self):
        top_100 = Current.top_100()
        no_cols = len(top_100.columns)
        self.assertEqual(no_cols, 9)


if __name__ == '__main__':
    unittest.main()



# needed libraries
from datetime import date, datetime
import pandas as pd
import requests
from coinsta.exceptions import WrongCoinCode

'''
        TODO
        - make logo
        - submit to zenodo
        - clean up setup code
'''


# Historical Class for all methods related to historical data
class Historical:
    """
    A  class that provides methods for scraping historical price data
    based on specified cryptocurrencies and time period from
    CoinMarketCap.
    """

    def __init__(self, ticker, start, end=None):
        """
        :start argument accepts a date object representing YYYYMMDD.
        :end argument accepts a date object representing YYYYMMDD.
        :ticker argument accepts strings representing ticker information
        """

        # check for mispecification
        if isinstance(start, date) is False:
            raise TypeError("Start argument must be a date object or strings with the alternative 'from_strings' "
                            "constructor")

        # convert start day into to appriopriate format
        start = start.isoformat().replace("-", "")

        # use today's date unless otherwise specified by user
        if end is None:
            today = date.today()
            formatted_today = today.isoformat().replace("-", "")
            end = formatted_today
        elif isinstance(end, date) is False:
            raise TypeError("End argument must be a date object or \
                                        strings with the alternative 'from_strings' constructor")
        else:
            end = end.isoformat().replace("-", "")

        # self assign default args 
        self.ticker = ticker
        self.start = start
        self.end = end

    def __repr__(self):
        return "<Historical({0}, {1}, {2})>".format(self.ticker, self.start, self.end)

    def __str__(self):
        return "Coinsta object: \n crypto_symbol: {0} \n start_period: {1} \n end_period: {2}".format(self.ticker,
                                                                                                      self.start,
                                                                                                      self.end)

        # f"Coinsta object: \n crypto_symbol: {self.ticker}" \
        # f"\n start_period: {self.start} \n end_period: {self.end}\n"

    def get_data(self):
        """
        This function scrapes and cleans the data of the specified tickers
        from CoinMarketCap website.
        """

        def ticker_checker(ticker):
            """
            This method shows all valid coin tickers used by
            the CoinMarketCap website. Users can use this method
            to know the name of the cryptocurrency of interest.
            """

            api_url = "https://api.coinmarketcap.com/v2/ticker/?limit=0"
            response = requests.get(api_url).json()
            coins_dict = response['data']

            for v in coins_dict.values():
                if v['symbol'] == ticker.upper():
                    return v['website_slug']
            raise WrongCoinCode('Invalid code from "coinmarketcap.com"')

        slug = ticker_checker(self.ticker)

        site_url = "https://coinmarketcap.com/currencies/{0}/historical-data/?start={1}&end={2}".format(slug,
                                                                                                        self.start,
                                                                                                        self.end)
        # f"https://coinmarketcap.com/currencies/{slug}/historical-data/?start={self.start}&end={self.end}"

        data = pd.read_html(site_url)
        df = data[0]
        df['Date'] = pd.to_datetime(df['Date'])

        df.rename(
            {
                "Open*": "Open",
                "Close**": "Close",
                "Market Cap": "Market_cap"
            },
            axis='columns', inplace=True
        )

        df.set_index('Date', drop=True, inplace=True)
        df.sort_index(inplace=True)
        return df

    @classmethod
    def from_strings(cls, ticker, start, end, hyphen=True):
        """
        An alternative constructor that accepts strings
        string format YYYY-MM-DD or YYYY/MM/DD. 
        Default format is the hyphen separate string: YYYY-MM-DD
        Turn arg `hyphen` to False if string to be parsed is: YYYY/MM/DD
        """

        if hyphen:
            start = datetime.strptime(start, "%Y-%m-%d").date()
            end = datetime.strptime(end, "%Y-%m-%d").date()
        else:
            start = datetime.strptime(start, "%Y/%m/%d").date()
            end = datetime.strptime(end, "%Y/%m/%d").date()

        return cls(ticker, start, end)


# Current class for all methods related to current cryptocurrency information
class Current:
    """
    A class that connects to CoinMarketCap API and returns current
    data information for specified ticker
    """

    def __init__(self):
        # check for the required presence of pandas package
        # replace with check for API read errors instead
        try:
            import pandas as pd
        except ImportError:
            pd = None

        if pd is None:
            raise NotImplementedError(
                "This package requires the installation of the 'pandas' package"
                "Install 'pandas' package with pip or conda"
            )
        else:
            pass

    @staticmethod
    def get_current(ticker):
        """
        A method the returns the current data on specified cryptocurrency tickers
        """

        def get_info():
            url = "https://api.coinmarketcap.com/v2/ticker/?limit=0"
            response = requests.get(url).json()

            coins_dict = response['data']

            for v in coins_dict.values():
                if v['symbol'] == ticker.upper():
                    my_dict = dict()
                    my_dict['name'] = v['name']
                    my_dict['symbol'] = v['symbol']
                    my_dict['rank'] = v['rank']
                    my_dict['circulating_supply'] = v['circulating_supply']
                    my_dict['total_supply'] = v['total_supply']
                    my_dict['max_supply'] = v['max_supply']

                    quotes = v['quotes']['USD']

                    for key, val in quotes.items():
                        my_dict[key] = val

                    return my_dict

            else:
                WrongCoinCode('Invalid code from "coinmarketcap.com"')

        info = get_info()

        return pd.DataFrame.from_dict(info, orient='index', columns=[info.get('name')])

    @staticmethod
    def global_info():
        """
        This method returns a dictionary on global information
        representing the status of all cryptocurrencies
        tracked by Coinmarketcap.
        """
        global_url = 'https://api.coinmarketcap.com/v2/global/'
        global_response = requests.get(global_url).json()

        for _, _ in global_response.items():
            glo_dict = dict()

            glo_dict['active_cryptos'] = global_response['data']['active_cryptocurrencies']
            glo_dict['active_markets'] = global_response['data']['active_markets']
            glo_dict['btc_dominance'] = global_response['data']['bitcoin_percentage_of_market_cap']

            glo_quotes = global_response['data']['quotes']['USD']

            for key, value in glo_quotes.items():
                glo_dict[key] = value

            return glo_dict  # pd.DataFrame.from_dict(glo_dict, orient='index')

    @staticmethod
    def top_100():
        """
        This method returns the name, ticker symbol and current price
        of the top 100 cryptocurrencies in terms of market capitalisation
        """
        api_url = "https://api.coinmarketcap.com/v2/ticker/?limit=0"
        top_response = requests.get(api_url).json()

        results_dict = {}

        for _, v in top_response['data'].items():

            container = dict()

            container['name'] = v['name']
            container['symbol'] = v['symbol']
            container['rank'] = v['rank']

            deep_nest = v['quotes']['USD']

            for key, value, in deep_nest.items():
                container[key] = value

            results_dict[container['name']] = container

        df = pd.DataFrame.from_dict(results_dict, orient='index').reset_index(drop=True)

        return df.sort_values('rank').reset_index(drop=True)


'''
class HistoricalSnapshot:
    """
    A class that returns a historical snapshot of Cryptocurrency market with information
    on the highest ranking cryptocurrencies.
    """
    pass

'''
